
#include "Arduino.h"
#define ENABLE 8
#define NECHANT 512
#define SHIFT_ACCUM 23
uint32_t icr;
uint32_t period_pwm;
uint32_t table_onde_A[NECHANT];
uint32_t table_onde_B[NECHANT];
uint32_t accum1,increm,indice;
uint16_t diviseur[6] = {0,1,8,64,256,1024};

void init_pwm_timer1(uint32_t period) {
    char clockBits;
    TCCR1A = 0;
    TCCR1A |= (1 << COM1A1); //Clear OC1A on compare match when upcounting, set OC1A on compare match when downcounting
    TCCR1A |= (1 << COM1B1); //Clear OC1B on compare match when upcounting, set OC1B on compare match when downcounting
    TCCR1B = 1 << WGM13; // phase and frequency correct pwm mode, top = ICR1
    int d = 1;
    icr = (F_CPU/1000000*period/2);
    while ((icr>0xFFFF)&&(d<6)) { // choix du diviseur d'horloge
        d++;
        icr = (F_CPU/1000000*period/2/diviseur[d]);
   } 
   clockBits = d;
   ICR1 = icr; // valeur maximale du compteur
   OCR1A = 0; // rapport cyclique
   OCR1B = 0;
   TIMSK1 = 1 << TOIE1; // overflow interrupt enable
   sei(); // activation des interruptions
   TCNT1 = 0; // mise à zéro du compteur
   TCCR1B |= clockBits; // déclenchement du compteur
}
               
ISR(TIMER1_OVF_vect) { // Timer 1 Overflow interrupt
  accum1 += increm;
  indice = accum1 >> SHIFT_ACCUM;
  OCR1A = table_onde_A[indice];
  OCR1B = table_onde_B[indice];
}
               
void set_table_harmoniques(float amp, float a1, float a3, float a5) {
    int i;
    float x;
    float dt = 2*3.1415926/NECHANT;
    for(i=0; i<NECHANT; i++) {
            x = amp*(a1*sin(i*dt)+a3*sin(3*i*dt)+a5*sin(5*i*dt));
            table_onde_A[i] = icr*0.5*(1+x);
            table_onde_B[i] = icr*0.5*(1-x);
        } 
}
               
void set_frequence(uint32_t frequence) {
    increm = (uint32_t) (((float)(0xFFFFFFFF))*((float)(frequence)*1e-6*(float)(period_pwm))); // incrément de l'accumulateur de phase
    
}
               
void setup() {
  pinMode(ENABLE,OUTPUT);
  digitalWrite(ENABLE,LOW);
  
  
#if defined(__AVR_ATmega2560__)
    pinMode(11,OUTPUT); // sortie PWM sur arduino MEGA
    digitalWrite(11,LOW);
    pinMode(12,OUTPUT);
    digitalWrite(12,LOW);
#else
    pinMode(9,OUTPUT); // sortie PWM sur arduino UNO ou YUN
    digitalWrite(9,LOW);
    pinMode(10,OUTPUT);
    digitalWrite(10,LOW);
#endif
    period_pwm = 40; // en microsecondes
    accum1 = 0;
    set_frequence(400);
    init_pwm_timer1(period_pwm);
    float amplitude = 0.1;
    set_table_harmoniques(amplitude,1.0,0.0,0.0);
    digitalWrite(ENABLE,HIGH);
}

void loop() {

}
               